/*  Copyright 2008 Peter Mora
 *
 *  This file is part of Vakarcs.
 *
 *  Vakarcs is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Vakarcs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "object.h"
#include <math.h>
#include "server.h"
#include "object.h"

Object::Object()
{
	//This is for test. In the final version we won't use this
}

Object::Object(Object* ob)	//copy
{
	m_id = ob->id();
	m_name = ob->name();
	m_red = ob->red();
	m_blue = ob->blue();
	m_white = ob->white();
	m_green = ob->green();
	m_black = ob->black();

	m_owner = ob->owner();
	m_posx = ob->x(); m_posy = ob->y();

	m_active = ob->active();
	m_visible= ob->visible();
	m_blocked = ob->blocked();
	for (int i = 0; i < COOLDOWN; i++)
		m_cooldown[i] = ob->cooldown()[i];
	for (int i = 0; i < 10; i++)
	{
		future_x[i] = 0; future_y[i] = 0;
		future_step[i] = "";
	}
}

Object::Object(string st)	//create from string
{
	vector<string>* r = tokenize(st,' ');
//	cout << "OK?\n";
//	cout << r->size() << "\n";
//	cout << st << "\n";
	if (r->size() == 12 + COOLDOWN)
	{
//		cout << "OK\n";
		m_id = atol((*r)[0].c_str());
		m_name = (*r)[1];
		m_red = atol((*r)[2].c_str());
		m_green = atol((*r)[3].c_str());
		m_blue = atol((*r)[4].c_str());
		m_white = atol((*r)[5].c_str());
//		m_black = atol((*r)[6].c_str());

		m_owner = atol((*r)[6].c_str());
		m_posx = atol((*r)[7].c_str());
		m_posy = atol((*r)[8].c_str());

		m_active = atol((*r)[9].c_str());
		m_visible= atol((*r)[10].c_str());
		m_blocked = atol((*r)[11].c_str());
		for (int i = 0; i < COOLDOWN; i++)
			m_cooldown[i] = atol((*r)[12+i].c_str());
	}
	delete r;
	for (int i = 0; i < 10; i++)
	{
		future_x[i] = 0; future_y[i] = 0;
		future_step[i] = "";
	}
}

string		Object::generate_string(int mirror, int maxx, int maxy)
{	
	int x = m_posx;
	int y = m_posy;
	int m = mirror;
	if (m >= 4)
	{
		int t = x;
		x = y;
		y = t;
		if (m == 4)
			m = 0;
		if (m == 5)
			m = 2;
		if (m == 6)
			m = 1;
		if (m == 7)
			m = 3;
	}
	string r = itostr(m_id)+string(" ")+
		m_name+string(" ")+
		itostr(m_red)+string(" ")+
		itostr(m_green)+string(" ")+
		itostr(m_blue)+string(" ")+
		itostr(m_white)+string(" ")+
//		itostr(m_black)+string(" ")+
		itostr(m_owner)+string(" ")+
		itostr(((m==0||m==2)?x:maxx-x-1))+string(" ")+
		itostr(((m==0||m==1)?y:maxy-y-1))+string(" ")+
		itostr(m_active)+string(" ")+
		itostr(m_visible)+string(" ")+
		itostr(m_blocked)+string(" ");
	for (int i = 0; i < COOLDOWN; i++)
	{
		r += itostr(m_cooldown[i])+string(" ") ;
	}
	return r;
}

Object::~Object()
{

}

void	Object::step_parameter(int* which)
{
	if (*which < 0)
	{
		*which = (*which) + 1;
		if (*which == 0)
			*which = 1;
	}
	if (*which > 1)
	{
		*which = (*which) - 1;
		if (*which == 1)
			*which = 0;
	}
}

string	Object::action(int part, Server* server, bool* redraw, PlayerInfo* own_playerinfo, vector<Ability*>* ab)
{
//	if (properties(type) <= 0)
//		return;

	if (part == 0)
	{
		for (int i = 0; i < COOLDOWN; i++)
			if (m_cooldown[i])
				m_cooldown[i]--;
		step_parameter(&m_active);
		step_parameter(&m_visible);
		if (m_blocked > 0)
			m_blocked --;
		return "";
	}
	if (m_blocked < 0 && part == 1)
	{
		m_step = string("ability") + itostr(-m_blocked);
//		cout << m_blocked << "\n";
	}
	if (part == 40)	//everybody sees themselves
	{
		*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,m_posx),0),MAX(MIN(server->maxy()-1,m_posy),0))) = MAPS_VISIBLE;
	}
	if (m_blocked > 0)
		return "";
	if (part == 99)
	{
		m_wall_change.clear();
	}

//	if (m_active<1)
//		return "";

	// FUSE
	if (part == 10 && m_step == "fuse")
	{
		int red = 0;
		int green = 0;
		int blue = 0;
		int white = 0;
		for (int q = 0; q < server->actual_round()->objects.size(); q++)
		{
			Object* ob = server->actual_round()->objects[q];
			if (ob->x() == m_posx && ob->y() == m_posy && ob->owner() == m_owner && ob->active() > 0)
			{
				red += ob->red();
				green += ob->green();
				blue += ob->blue();
				white += ob->white();
			}
		}
		if (white == m_white && green == m_green && blue == m_blue && red == m_red)
		{
			own_playerinfo->strings.push_back(m_name + " vakarccsal senki sem áll ugyanazon a mezőn.");
		}
		else
		{
			if (( white == 0 && red <= MAX_VAKARCS && green <= MAX_VAKARCS && blue <= MAX_VAKARCS) || ( red == 0 && green == 0 && blue == 0 && white <= 2*MAX_VAKARCS))
			{
				for (int q = 0; q < server->actual_round()->objects.size(); q++)
				{
					Object* ob = server->actual_round()->objects[q];
					if (ob->x() == m_posx && ob->y() == m_posy && ob->owner() == m_owner && ob->id() != m_id && ob->active() > 0)
					{
						ob->active(0);
					}
				}
				for (int i = 0; i < COOLDOWN; i++)
					m_cooldown[i] = 0;
				own_playerinfo->strings.push_back(m_name + " sikeresen összeépült a vele egy mezőn álló vakarcsokkal.");
				m_red = red; m_green = green; m_blue = blue; m_white = white;
			}
			else
				own_playerinfo->strings.push_back(m_name + " nem tudott összeépülni a vele egy mezőn álló vakarcsokkal.");// A mezőn lévő vakarcsok: "+itostr(red) + " " +itostr(green) + " " + itostr(blue) + " " + itostr(white) +".");
		}

		redraw[m_owner] = true;


	}
	// MOVE
	if(part == 20 && m_step == "move")
	{
		if (properties(m_step) == 1 && abs(m_stepx)+abs(m_stepy) == 1 && m_posx+m_stepx >=0 && m_posy+m_stepy >=0 && m_posx+m_stepx< server->maxx() && m_posy+m_stepy < server->maxy() && (server->map(m_posx+m_stepx,m_posy+m_stepy) != MAP_WALL || has_ability("levitation",ab)))
		{
			m_posx += m_stepx;
			m_posy += m_stepy;
			if (server->map(m_posx,m_posy) >= MAP_BASE && server->map(m_posx,m_posy) != MAP_BASE+m_owner && !has_ability("cantwin",ab) && server->map(m_posx,m_posy)-MAP_BASE < server->number_of_players() && server->active_players()[server->map(m_posx,m_posy)-MAP_BASE])
			{

				m_active = 0;
				own_playerinfo->strings.push_back(m_name + " beért az ellenfél bázisára.");
				(*server->next_player_info())[server->map(m_posx,m_posy)-MAP_BASE]->strings.push_back(m_name + " vakarcs beért a bázisodra!");
				server->score(server->map(m_posx,m_posy)-MAP_BASE,1);
			}
//			return string("");
		}
		else
		{
			own_playerinfo->strings.push_back(m_name + ": nem tudott lépni.");	
//			return string("unable to move");
		}
	}

	// SEE
	if(part == 40)
	{
		int s = properties("see");
		for (int x = -s; x <= +s; x++)
		{
			for (int y = 0; y <= s; y++)
			{
				int xx = m_posx + x*y/s;
				int yy = y + m_posy;
				*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,xx),0),MAX(MIN(server->maxy()-1,yy),0))) = MAPS_VISIBLE;
				if (xx < 0 || yy < 0 || xx>= server->maxx() || yy >= server->maxy() || (server->map(xx,yy) == MAP_WALL && !has_ability("levitation",ab)))
					break;
			}
			for (int y = 0; y <= s; y++)
			{
				int xx = m_posx + x*y/s;
				int yy = -y + m_posy;
				*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,xx),0),MAX(MIN(server->maxy()-1,yy),0))) = MAPS_VISIBLE;
				if (xx < 0 || yy < 0 || xx>= server->maxx() || yy >= server->maxy() || (server->map(xx,yy) == MAP_WALL && !has_ability("levitation",ab)))
					break;
			}
		}
		for (int y = -s; y <= +s; y++)
		{
			for (int x = 0; x <= s; x++)
			{
				int yy = m_posy + x*y/s;
				int xx = x + m_posx;
				*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,xx),0),MAX(MIN(server->maxy()-1,yy),0))) = MAPS_VISIBLE;
				if (xx < 0 || yy < 0 || xx>= server->maxx() || yy >= server->maxy() || (server->map(xx,yy) == MAP_WALL && !has_ability("levitation",ab)))
					break;
			}
			for (int x = 0; x <= s; x++)
			{
				int yy = m_posy + x*y/s;
				int xx = -x + m_posx;
				*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,xx),0),MAX(MIN(server->maxy()-1,yy),0))) = MAPS_VISIBLE;
				if (xx < 0 || yy < 0 || xx>= server->maxx() || yy >= server->maxy() || (server->map(xx,yy) == MAP_WALL && !has_ability("levitation",ab)))
					break;
			}
		}
		redraw[m_owner] = true;
	}
//	if (part == 60)
//		cout << m_step << "\n";
	if(m_step.substr(0,7) == "ability")
	{
		return ability(m_step[7]-'0', part, server, redraw, own_playerinfo,ab);
	}

	return string("");
}

int	Object::properties(string type, int red, int blue, int white, int green, int black)
{
	int r = red; int b = blue; int w = white; int g = green; int bl = black;
	if (red == -1)
	{
		r = m_red; b = m_blue; w = m_white; g = m_green; bl = m_blue;
	}
	if (type == "move")
		return 1;

	if (type == "see")
	{
		return 3;
	}
//	if (type == "ability0")
//	{
//		return ABILITY_SHOOT;
//	}
//	if (type == "shoot")
//	{
//		return 3;
//	}

	return -1;
}

string	Object::ability(int ab_num_from_string, int part, Server* server, bool* redraw, PlayerInfo* own_playerinfo, vector<Ability*>* ab )
{
//	if (part == 60)
//		cout << ".";
	int a = 0;
	for (;a<ab->size();a++)
	{
		if ((*ab)[a]->red == m_red && (*ab)[a]->green == m_green && (*ab)[a]->blue == m_blue && (*ab)[a]->white == m_white)
			break;
	}
	if (a < ab->size())
	{
//		cout << a << "\n" << ab_num << "\n";
//		cout << (*ab)[a]->ability[ab_num] << "\n";
//		cout << (*ab)[a]->ability.size() << "\n";
//		cout << "-------" << "\n";
//		cout.flush();
		int ab_num = (*ab)[a]->number[ab_num_from_string];
		if (-1 == ab_num)
		{
			if (part == 40)
			{
				own_playerinfo->strings.push_back(m_name + ": nincs " + itostr(ab_num_from_string) + ". képesség.");
				redraw[m_owner] = true;
				return string("no ability found");
			}
			else
				return "";
		}	
		int time = 60;
		if ((*ab)[a]->ability[ab_num]=="block")
			time = 5;
		if ((*ab)[a]->ability[ab_num]=="shoot")
			time = 60;
		if ((*ab)[a]->ability[ab_num]=="bomb")
			time = 60;
		if ((*ab)[a]->ability[ab_num]=="blink")
			time = 20;
		if ((*ab)[a]->ability[ab_num]=="random-sight")
			time = 40;
		if ((*ab)[a]->ability[ab_num]=="nearest")
			time = 41;
		if ((*ab)[a]->ability[ab_num]=="magicattack")
			time = 60;
		if ((*ab)[a]->ability[ab_num]=="magicshoot")
			time = 60;
		if ((*ab)[a]->ability[ab_num]=="flood")
			time = 60;
		if ((*ab)[a]->ability[ab_num]=="wall")
			time = 80;
		if ((*ab)[a]->ability[ab_num]=="inverse")
			time = 80;
		if ((*ab)[a]->ability[ab_num]=="timeback")
			time = 90;
		if (((*ab)[a]->ability[ab_num]=="wall" ||(*ab)[a]->ability[ab_num]=="inverse")&& part == 81 )
		{
			for (int i = 0; i < m_wall_change.size(); i++)
			{
				server->actual_round()->map[m_wall_change[i]] = m_wall_change_to;
				*(server->own_visible_map(m_owner,m_wall_change[i]%server->maxx(),m_wall_change[i]/server->maxx())) = MAPS_VISIBLE;
				redraw[m_owner] = true;
			}
			m_wall_change.clear();
		}

		if (part != time)
			return "";
		if (m_cooldown[ab_num])
		{
			own_playerinfo->strings.push_back(m_name + ": a képesség még nem töltött újra.");	
			redraw[m_owner] = true;
			return string("cooldown problem");
		}
		// SHOOT
		if ((*ab)[a]->ability[ab_num] == "shoot")
		{
			if (abs(m_stepx)+abs(m_stepy) == 1)
			{
				m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
				int range = (*ab)[a]->parameter[ab_num];
				//bool bo = false;
				for (int i = 1; i < range+1; i++)
				{
					if (m_posx+i*m_stepx < 0 || m_posy+i*m_stepy < 0 || m_posx+i*m_stepx>= server->maxx() || m_posy+i*m_stepy >= server->maxy() || server->map(m_posx+i*m_stepx,m_posy+i*m_stepy) == MAP_WALL)
						break;
					destroy(redraw,m_posx+i*m_stepx, m_posy+i*m_stepy, server,own_playerinfo,m_name + " lelötte "," vakarcsot.",m_name + " lelötte "," vakarcsodat.",1);
					redraw[m_owner]=true;
				}
//				own_playerinfo->strings.push_back("Senkit sem találtunk el.");
				return string("");
			}

		}
		// BOMB
		if ((*ab)[a]->ability[ab_num] == "bomb")
		{
			m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
			for (int x = -1; x < 2; x++)
				for (int y = -1; y < 2; y++)
					if(m_posx+x >= 0 && m_posx+x < server->maxx() && m_posy+y>=0 && m_posy+y<server->maxy())
					{
						destroy(redraw,m_posx+x, m_posy+y, server,own_playerinfo,m_name + " felrobbantotta "," vakarcsot.",m_name + " felrobbantotta "," vakarcsodat.",2,this);
						redraw[m_owner]=true;
					}
			return string("");

		}
		// MAGICSHOOT
		if ((*ab)[a]->ability[ab_num] == "magicshoot")
		{
			if (m_stepx < server->maxx() && m_stepy < server->maxy() && m_stepx >= 0 && m_stepy >= 0)
			{
				int range = (*ab)[a]->parameter[ab_num];
				if (abs(m_stepx - m_posx) <= range && abs(m_stepy - m_posy) <= range)
				{
					m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
					destroy(redraw,m_stepx, m_stepy, server,own_playerinfo,m_name + " mágialövéssel lelötte "," vakarcsot.",m_name + " mágialövéssel lelötte "," vakarcsodat.",1);
					redraw[m_owner]=true;
					return string("");
				}
				else
				{
					own_playerinfo->strings.push_back(m_name + ": nem tud olyan messszire lőni.");
					return string("");
				}
			}

		}
		// BLINK
		if ((*ab)[a]->ability[ab_num] == "blink")
		{
				int range = (*ab)[a]->parameter[ab_num];
				if (m_posx+m_stepx*range >=0 && m_posy+m_stepy*range >=0 && m_posx+m_stepx*range< server->maxx() && m_posy+m_stepy*range < server->maxy() )
				{
					if (server->map(m_posx+m_stepx*range,m_posy+m_stepy*range) == MAP_WALL)
					{
						own_playerinfo->strings.push_back(m_name + " falba ugrott.");
						m_active = 0;
					}
					m_posx += m_stepx*range;
					m_posy += m_stepy*range; 
					if (server->map(m_posx,m_posy) >= MAP_BASE && server->map(m_posx,m_posy) != MAP_BASE+m_owner && server->map(m_posx,m_posy)-MAP_BASE < server->number_of_players() && server->active_players()[server->map(m_posx,m_posy)-MAP_BASE])
					{
						m_active = 0;
						own_playerinfo->strings.push_back(m_name + " beért az ellenfél bázisára.");
						(*server->next_player_info())[server->map(m_posx,m_posy)-MAP_BASE]->strings.push_back(m_name + " vakarcs beért a bázisodra!");
						server->score(server->map(m_posx,m_posy)-MAP_BASE,1);
					}
					m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
					return string("");
				}
				else
				{
					own_playerinfo->strings.push_back(m_name + ": nem tudott lépni.");	
					return string("unable to move");
				}
		}
		// RANDOM-SIGHT
		if ((*ab)[a]->ability[ab_num] == "random-sight")
		{
			int range = (*ab)[a]->parameter[ab_num];
			int x = rand() % (server->maxx()-range);
			int y = rand() % (server->maxy()-range);
			for (int xx = x; xx < x+range; xx++)
				for (int yy = y; yy < y+range; yy++)
					*(server->own_visible_map(m_owner,MAX(MIN(server->maxx()-1,xx),0),MAX(MIN(server->maxy()-1,yy),0))) = MAPS_VISIBLE;

		}
		// BLOCK
		if ((*ab)[a]->ability[ab_num] == "block")
		{
			m_blocked = -ab_num_from_string;
			block(server,(*ab)[a]->parameter[ab_num]);
		}
		// TIMEBACK
		if ((*ab)[a]->ability[ab_num] == "timeback")
		{
			int r = server->timeback((*ab)[a]->parameter[ab_num],(*ab)[a]->cooldown[ab_num],m_owner);
			for (int i = 0; i < server->number_of_players(); i++)
				(*server->next_player_info())[i]->strings.push_back(m_name + " vakarcs visszapörgette az időt "+itostr(r)+" körrel.");
		}
		// MAGICATTACK
		if ((*ab)[a]->ability[ab_num] == "magicattack")
		{
			if (abs(m_stepx)+abs(m_stepy) == 1)
			{
				m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
				int wall = (*ab)[a]->parameter[ab_num];
				int range;
				for (range = 1; true; range++)
				{
					if (m_posx+range*m_stepx < 0 || m_posy+range*m_stepy < 0 || m_posx+range*m_stepx>= server->maxx() || m_posy+range*m_stepy >= server->maxy() )
						break;
					if (server->map(m_posx+range*m_stepx,m_posy+range*m_stepy) == MAP_WALL)
					{
						if (--wall < 0)
							break;
					}
				}
				for (int i = 1; i < range; i++)
				{
					if (server->actual_round()->map[(m_posy+i*m_stepy)*server->maxx()+m_posx+i*m_stepx] == MAP_EMPTY)
						server->actual_round()->map[(m_posy+i*m_stepy)*server->maxx()+m_posx+i*m_stepx] = MAP_DESTROYED;
					destroy(redraw,m_posx+i*m_stepx, m_posy+i*m_stepy, server,own_playerinfo,m_name + " tűzszőnyeggel lelötte "," vakarcsot.",m_name + " tűzszőnyeggel lelőtte "," vakarcsodat.",2);
				}
			}
		}
		// FLOOD
		if ((*ab)[a]->ability[ab_num] == "flood")
		{
			if (server->map(m_posx,m_posy) == MAP_WALL)
			{
				own_playerinfo->strings.push_back(m_name + ": nem lehet falról árasztani.");
				return "";
			}
			vector<int> xx;
			vector<int> yy;
			xx.push_back(m_posx);
			yy.push_back(m_posy);
			int from = 0;
			int to = 1;

			while(from != to)
			{
				for (int i = from; i < to; i++)
				{
					for (int j = 0; j < 4; j++)
					{
						int x = xx[i]; int y = yy[i];
						switch (j){
							case 0:
								x++;
								break;
							case 1:
								x--;
								break;
							case 2:
								y++;
								break;
							case 3:
								y--;
								break;
						}
						if(x >= 0 && x < server->maxx() && y >= 0 && y < server->maxy() && server->map(x,y) != MAP_WALL)
						{
							bool b = true;
							for (int q = 0; q < xx.size(); q++)
								if (xx[q] == x && yy[q] == y)
								{
									b = false;
									break;
								}
							if (b)
							{
								xx.push_back(x);
								yy.push_back(y);
							}
						}
					}
				}
				from = to;
				to = xx.size();
			}
			for (int q = 0; q < to; q++)
				destroy(redraw,xx[q],yy[q], server,own_playerinfo,m_name + " elárasztotta a pályát és "," vakarcs áladozatul esett.",m_name + " elárasztotta a pályát és "," vakarcsod áldozatul esett.",3);
		}
		// WALL
		if ((*ab)[a]->ability[ab_num] == "wall")
		{
			if (abs(m_stepx)+abs(m_stepy) == 1)
			{
				if (m_posx+m_stepx < 0 || m_posy+m_stepy < 0 || m_posx+m_stepx>= server->maxx() || m_posy+m_stepy >= server->maxy())
				{
						own_playerinfo->strings.push_back(m_name + ": a pályán belül kellene építkezni.");
						return "";
				}
				
				for (int q = 0; q < server->actual_round()->objects.size(); q++)
				{
					Object* ob = server->actual_round()->objects[q];
					if (ob->active() > 0 && !ob->has_ability("levitation",ab) && ob->x() == m_posx+m_stepx && ob->y() == m_posy+m_stepy )
					{
						own_playerinfo->strings.push_back(m_name + ": a falépítés nem jött össze, áll ott valaki.");
						return "";
					}
				}
				m_wall_change_to = -100;
				if (server->map(m_posx+m_stepx,m_posy+m_stepy) == MAP_WALL)
					m_wall_change_to = 0;
				if (server->map(m_posx+m_stepx,m_posy+m_stepy) == 0 || server->map(m_posx+m_stepx,m_posy+m_stepy) == MAP_DESTROYED)
					m_wall_change_to = MAP_WALL;
				if (m_wall_change_to != -100)
				{
					m_wall_change.push_back(server->maxx()*(m_posy+m_stepy)+m_posx+m_stepx);
					m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
				}
				else
				{
					own_playerinfo->strings.push_back(m_name + ": a falépítés nem jött össze.");
				}

			}
		}
		// INVERSE
		if ((*ab)[a]->ability[ab_num] == "inverse")
		{
			if (abs(m_stepx)+abs(m_stepy) == 1)
			{
				if (m_posx+m_stepx < 0 || m_posy+m_stepy < 0 || m_posx+m_stepx>= server->maxx() || m_posy+m_stepy >= server->maxy())
				{
						own_playerinfo->strings.push_back(m_name + ": a pályán belül kellene építkezni.");
						return "";
				}
				
				for (int q = 0; q < server->actual_round()->objects.size(); q++)
				{
					Object* ob = server->actual_round()->objects[q];
					if (ob->active() > 0 && ob->x() == m_posx+m_stepx && ob->y() == m_posy+m_stepy )
					{
						own_playerinfo->strings.push_back(m_name + ": a falépítés nem jött össze, áll ott valaki.");
						return "";
					}
				}
				m_wall_change_to = -100;
				if (server->map(m_posx+m_stepx,m_posy+m_stepy) == MAP_WALL)
					m_wall_change_to = 0;
				if (server->map(m_posx+m_stepx,m_posy+m_stepy) == 0 || server->map(m_posx+m_stepx,m_posy+m_stepy) == MAP_DESTROYED)
					m_wall_change_to = MAP_WALL;
				if (m_wall_change_to != -100)
				{
					m_wall_change.push_back(server->maxx()*(m_posy+m_stepy)+m_posx+m_stepx);
					m_cooldown[ab_num] = (*ab)[a]->cooldown[ab_num];
					for (int i = 2; true; i++)
					{
						if (m_posx+m_stepx*i < 0 || m_posy+m_stepy*i < 0 || m_posx+m_stepx*i>= server->maxx() || m_posy+m_stepy*i >= server->maxy())
							return "";
						if (server->map(m_posx+m_stepx*i,m_posy+m_stepy*i) >= MAP_BASE)
							return "";
						if (server->map(m_posx+m_stepx*i,m_posy+m_stepy*i) == MAP_WALL && m_wall_change_to == MAP_WALL)
							return "";
						if ((server->map(m_posx+m_stepx*i,m_posy+m_stepy*i) == 0 || server->map(m_posx+m_stepx*i,m_posy+m_stepy*i) == MAP_DESTROYED) && m_wall_change_to == 0)
							return "";
						bool stop = false;
						for (int q = 0; q < server->actual_round()->objects.size(); q++)
						{
							Object* ob = server->actual_round()->objects[q];
							if (ob->active() > 0 && ob->x() == m_posx+m_stepx && ob->y() == m_posy+m_stepy )
								return "";
						}

						m_wall_change.push_back(server->maxx()*(m_posy+m_stepy*i)+m_posx+m_stepx*i);
					}
				}
				else
				{
					own_playerinfo->strings.push_back(m_name + ": a falépítés nem jött össze.");
				}

			}
		}
		// NEAREST
		if ((*ab)[a]->ability[ab_num] == "nearest")
		{
			int r = 10000000;
			for (int q = 0; q < server->actual_round()->objects.size(); q++)
			{
				Object* ob = server->actual_round()->objects[q];
				int t;
				if (ob->active() > 0 && ob->owner() != m_owner && r > (t = (int)sqrt((ob->x()-m_posx)*(ob->x()-m_posx)+(ob->y()-m_posy)*(ob->y()-m_posy))))
					r = t;
			}
			if (r == 10000000)
				own_playerinfo->strings.push_back("Az ellenfélnek nincs élő vakarcsa.");
			else
			{
				int xx = m_posx;
				int yy = m_posy;
				int m = server->mirror(m_owner);
				if (m >= 4)
				{
					int t = xx;
					xx = yy;
					yy = t;
					if (m == 4)
						m = 0;
					if (m == 5)
						m = 2;
					if (m == 6)
						m = 1;
					if (m == 7)
						m = 3;
				}
				own_playerinfo->strings.push_back( string("--CIRCLE(") + itostr(((m==0||m==2)?xx:server->maxx()-xx-1)) + ","+itostr(((m==0||m==1)?yy:server->maxy()-yy-1))+","+itostr(r)+")--A legközelebbi ellenséges vakarcs távolsága: "+ itostr(r) + " egység.");
/*					for (int x = 0; x < server->maxx(); x++)
									for (int y = 0; y < server->maxy(); y++)
									if( ((int)sqrt((x-m_posx)*(x-m_posx)+(y-m_posy)*(y-m_posy))) == r && *(server->own_visible_map(m_owner,x,y) < MAPS_MARKED)
									{
				 *(server->own_visible_map(m_owner,x,y) += MAPS_MARKED;
				 }*/
			}
			redraw[m_owner] = true;
		}
	}
	else
	{
		cout << "ERROR\n";
		return "error, not enough specification";
	}

	return string("unknown ability");
}

bool	Object::has_ability(string st, vector<Ability*>* ab)
{
	int a = 0;
	for (;a<ab->size();a++)
	{
		if ((*ab)[a]->red == m_red && (*ab)[a]->green == m_green && (*ab)[a]->blue == m_blue && (*ab)[a]->white == m_white)
			break;
	}
	if (a < ab->size())
	{
		for (int i = 0; i < (*ab)[a]->ability.size(); i++)
			if (st == (*ab)[a]->ability[i])
				return true;
	}
	return false;
}

int	Object::ability_parameter(string st, vector<Ability*>* ab)
{
	int a = 0;
	for (;a<ab->size();a++)
	{
		if ((*ab)[a]->red == m_red && (*ab)[a]->green == m_green && (*ab)[a]->blue == m_blue && (*ab)[a]->white == m_white)
			break;
	}
	if (a < ab->size())
	{
		for (int i = 0; i < (*ab)[a]->ability.size(); i++)
			if (st == (*ab)[a]->ability[i])
				return (*ab)[a]->parameter[i];
	}
	return 0;
}

//damage = 1 -> single shot
//damage = 2 -> visible for everybody, destroyes walls
//damage = 2 -> visible for everybody, destroyes walls, kills vakarcs with bulletproof as well
void	Object::destroy(bool* redraw,int x, int y,Server* server, PlayerInfo* own_playerinfo, string own_before, string own_after, string target_before, string target_after, int damage, Object* avoid)
{
	if (damage >= 2 && (server->map(x,y) == 0 || server->map(x,y) == MAP_WALL))
	{
		server->actual_round()->map[server->maxx()*y+x] = MAP_DESTROYED;
		for (int q = 0; q< server->number_of_players(); q++)
			redraw[q] = true;
	}
	vector<Object*>	will_bomb;
	for (int q = 0; q < server->actual_round()->objects.size(); q++)
	{
		Object* ob = server->actual_round()->objects[q];
		if (ob->active() > 0 && ob->x() == x && ob->y() == y && (!ob->has_ability("bulletproof",server->abilities()) || damage >= 3) && ob != avoid)
		{
			ob->active(0);
			if (m_owner != ob->owner())
				own_playerinfo->strings.push_back(own_before + ob->name() +own_after);
			(*server->next_player_info())[ob->owner()]->strings.push_back(target_before + ob->name() + target_after);
			if (ob->has_ability("bomb-on-death",server->abilities()))
			{
				if (m_owner != ob->owner())
					own_playerinfo->strings.push_back(ob->name() + " felrobbant.");
				(*server->next_player_info())[ob->owner()]->strings.push_back(ob->name() + " vakarcsod felrobbant.");
				will_bomb.push_back(ob);
			}
			redraw[ob->owner()] = true;
		}
	}
	for (int i = 0; i < will_bomb.size(); i++)
	{
		will_bomb[i]->bomb_on_death(will_bomb[i]->ability_parameter("bomb-on-death",server->abilities()),server,redraw);
	}
}

void	Object::bomb_on_death(int range, Server* server, bool* redraw)
{
	for (int i = -range/2; i <= range/2; i++)
	for (int j = -range/2; j <= range/2; j++)
	{
		if (m_posx+i < 0 || m_posy+j < 0 || m_posx+i>= server->maxx() || m_posy+j >= server->maxy())
			continue;
		destroy(redraw, m_posx+i, m_posy+j, server,(*server->next_player_info())[m_owner],m_name + " felrobbantotta "," vakarcsot.",m_name + " felrobbantotta "," vakarcsodat.",2);
	}
}
void	Object::block(Server* server,int range)
{
	for (int q = 0; q < server->actual_round()->objects.size(); q++)
	{
		Object* ob = server->actual_round()->objects[q];
		if (ob->active() > 0 && abs(ob->x() - m_posx) <= range/2 && abs(ob->y()-m_posy) <= range/2 && ob->blocked() <= 1 && ob->blocked() >= 0 && ob->x() >= 0 && ob->y() >= 0 && ob->x() < server->maxx() && ob->y() < server->maxy() && server->map(ob->x(),ob->y()) < MAP_BASE)
		{
			ob->blocked(2);
			(*server->next_player_info())[ob->owner()]->strings.push_back(ob->name() + " vakarcsodat blokkolták.");
//			*(server->own_visible_map(ob->owner(),MAX(MIN(server->maxx()-1,m_posx),0),MAX(MIN(server->maxy()-1,m_posy),0))) = MAPS_VISIBLE;
		}
	}
}
