/*  Copyright 2008 Peter Mora
 *
 *  This file is part of Vakarcs.
 *
 *  Vakarcs is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Vakarcs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _UI_H
#define _UI_H
#include "server.h"
#include <locale.h>
#ifdef __WIN32__
 #include "curses.h"
#else
 #include <ncursesw/ncurses.h>
#endif


#define MAP_OFFSET 1

struct Translate{
	string	name;
	string	longname;
	string	shortname;
	string	parameter;
	string	description;
};

class Ui{
	string			m_abilities_types;

	vector<Translate*>	m_translate;
	string			m_t_cooldown;
	string			m_t_parameter;
	string			m_t_help;
	string			m_t_credits;
//	Server	m_server;
	vector<PlayerInfo*>	m_player_info;

	vector<int>		m_sent_positions;

	int	m_mode;		// 0 : map
				// 1 : vak_menu
				// 2 : direction?
				// 3 : position?
	
	int	m_set_ability_round;
	string	m_set_ability_string;
	int	m_set_ability_from_x;
	int	m_set_ability_from_y;
	int	m_set_ability_x;
	int	m_set_ability_y;

	int	m_vak_red;
	int	m_vak_green;
	int	m_vak_blue;
	int	m_vak_white;

	int	m_maxx;
	int	m_maxy;

	bool	m_game_over;

//	int	m_last_round;

	int	m_current_round;
	int	m_current_info;
	int	m_current_info_string;
	string	m_current_message;

	int	m_chosen_object;

	int	m_wall_type;

	void	draw_map();

	void	make_move(int x, int y);	
	Object*	find();
	void	color_print(int y, int x, const char* st, int fore, int back);
	void	info_print(const char* st, int fore = COLOR_WHITE, int back = COLOR_BLACK);

	int	m_info_print_line;
	int	m_info_print_pos;
	vector<int>	m_color_fore;
	vector<int>	m_color_back;
	int	m_player_id;
	int	m_number_of_players;

	vector<Ability*>	m_abilities;

	void	set_ability(int x, int y);
	int	find_translate(string str);
	void	desc_print(int *line, string str);
	Ability*	find_ability(int red, int green, int blue, int white);
	void	position(Object* ob, int &x, int &y);

	void	next_step();
	void	prev_step();
public:
	Ui();
	~Ui();
	void	message (std::string);
	std::string	play();
	void	help();
	void	credits();
	void	setxy(int x, int y){m_maxx = x; m_maxy = y;};
	void	set_abilities(string str);
	void	set_player_id(int id, int n){m_player_id = id;m_number_of_players = n;}
	void	load_language(string str);
};
#endif
